@@ -5,7 +5,6 @@ import math
 import argparse
 import os
 import sys
-import heapq
 from pkg_resources import resource_filename
 
 # The dictionary bundled with the script. It must be in the same
@@ -15,15 +14,20 @@ DEFAULT_DICT =  resource_filename(__name__, 'dict.txt')
 #DEFAULT_DICT="/usr/share/dict/words"
 WORDS = 4
 BAD_CH_LIST = ['\'']
-VERSION_NUMBER="0.1.1"
+VERSION_NUMBER="0.1.2"
 
 class WordValidator(object):
+    """Validates words from the dictionary file, when the file is loaded
+       into memory.
+    """
     def __init__(self, flags):
-        self._badChars = BAD_CH_LIST
+        self._badChars = set(BAD_CH_LIST)
         self._maxLen = flags.l
         self._skipBad = flags.x
 
     def isValidWord(self, word):
+        """Returns True if the word is valid. False otherwise.
+        """
         if len(word) > self._maxLen or len(word) == 0:
             return False
         if self._skipBad and self._hasBadChar(word):
@@ -31,7 +35,9 @@ class WordValidator(object):
         return True
 
     def _hasBadChar(self, word):
-        for char in word:
+        """Return true if word has a 'bad' char.
+        """
+        for char in reversed(word):
             if char in self._badChars:
                 return True
 
@@ -52,6 +58,8 @@ class Dictionary(object):
         return dict_file
 
 class DictionaryList(Dictionary):
+    """List-like object that returns words from a dictionary file.
+    """
     def __init__(self, flags):
         super(DictionaryList, self).__init__(flags)
         self._loadDict(flags.d)
@@ -63,6 +71,8 @@ class DictionaryList(Dictionary):
         return self._dictLen
 
     def _loadDict(self, path):
+        """Load every word from dict_file that passes the isValidWord test.
+        """
         self._dictList = []
         dict_file = self._openDict()
         for line in dict_file:
@@ -73,6 +83,8 @@ class DictionaryList(Dictionary):
         dict_file.close()
 
 class RandomDict(object):
+    """Iterator that returns on random word per iteration.
+    """
     def __init__(self, flags):
         self._dictObj = DictionaryList(flags)
         self._rng = random.SystemRandom()
@@ -90,6 +102,8 @@ class RandomDict(object):
         return getattr(self._dictObj, attr)
 
 class PasswordGen(object):
+    """Iterator that returns one password per iteration.
+    """
     def __init__(self, flags):
         self._randSource = RandomDict(flags)
         self._wordCount = flags.w
@@ -100,20 +114,55 @@ class PasswordGen(object):
 
     def next(self):
         rand_source_iter = iter(self._randSource)
-        word_list = [rand_source_iter.next() for i in range(self._wordCount)]
+        word_list = [rand_source_iter.next() for i in xrange(self._wordCount)]
         return self._delimiter.join(word_list)
 
     def getInfo(self):
-        pos = len(self._randSource) ** self._wordCount
-        entropy = math.log(pos, 2)
-        yrs_to_crack = pos/(60.*60*24*365*1000000)
+        entropy, entr_per_word = self.calcEntropy()
         info =  "\nInfo:\n"
         info += "  Entropy: %0.3f bits\n" % entropy
-        info += "  Entropy per word: %0.3f bits\n" % (entropy / self._wordCount)
-        info += "  At 1 million tries per second, it would take at most %0.3f years to crack.\n" % yrs_to_crack
+        info += "  Entropy per word: %0.3f bits\n" % entr_per_word
+        info += "  Possible combinations given settings: %s\n" % self.readableNum(self.calcPos())
+        for triespersec in [1e3, 71e3, 77e6, 348e9]:
+            info += self.makeYrsToCrackMsg(triespersec)
         return info
 
+    def calcEntropy(self):
+        entropy = math.log(self.calcPos(), 2)
+        return entropy, entropy/self._wordCount
+
+    def calcYrsToCrack(self, triesPerSec):
+        try:
+            return (self.calcPos()/(60*60*24*365.25*triesPerSec))/2
+        except OverflowError:
+            # If float calculation overflows, estimate using long.
+            return (self.calcPos()/(60*60*24*365*int(triesPerSec)))/2
+
+    def makeYrsToCrackMsg(self, triesPerSec):
+        try:
+            return "  Average time to crack at %s tries per second: %s years\n" % (self.readableNum(triesPerSec), self.readableNum(self.calcYrsToCrack(triesPerSec)))
+        except TypeError:
+            # If float conversion overflows, use long.
+            return "  Average time to crack at %s tries per second: %s years\n" % (self.readableNum(triesPerSec), self.calcYrsToCrack(triesPerSec))
+
+    def calcPos(self):
+        return len(self._randSource) ** self._wordCount
+
+    def readableNum(self, num, postfix=""):
+        num_words = { 1000000000000: 'trillion ',
+                      1000000000   : 'billion ',
+                      1000000      : 'million ',
+                      1000         : 'thousand ', }
+        for div, word in num_words.items():
+            if num/div >= 1:
+                return self.readableNum(num/div, word + postfix)
+        return "{:,.1f} {:s}".format(num, postfix).rstrip()
+
+
 class Enumerator(object):
+    """Accepts an iterator and prepends 1., 2., 3., etc to the beginning
+       of each item returned from the generator.
+    """
     def __init__(self, gen):
         self.gen = gen
         self.count = 0
@@ -125,58 +174,82 @@ class Enumerator(object):
         self.count += 1
         return "%d. %s" % (self.count, self.gen.next())
 
+
 def createParser():
+    """Create the parser for the command line flagsi
+    """
+    # Default values for command line flags.
+    flag_defaults = {
+        'w': WORDS,
+        'n':False,
+        'd':DEFAULT_DICT,
+        'x':True,
+        'i':False,
+        's':'-',
+        'l':100,
+        'c':1,
+        'v':False,
+        }
     parser = argparse.ArgumentParser(
             description='Generate an xkcd style password.',
             epilog='http://xkcd.com/936/')
     parser.add_argument('w',
             type=int,
-            default=WORDS,
+            default=flag_defaults['w'],
             nargs='?',
             help="The number of words in the password. Defaults to 4.")
     parser.add_argument('-n',
             action='store_true',
-            default=False,
+            default=flag_defaults['n'],
             help="Disable printing a newline at the end of the password.\
                   Good for piping to the clipboard.")
     parser.add_argument('-d',
-            default=DEFAULT_DICT,
+            default=flag_defaults['d'],
             metavar="DICT_PATH",
             help="The dictionary file. Defaults to %s." % DEFAULT_DICT)
     parser.add_argument('-x',
             action='store_false',
-            default=True,
+            default=flag_defaults['x'],
             help="Disable excluding special characters and punctuation.")
     parser.add_argument('-i',
             action='store_true',
-            default=False,
+            default=flag_defaults['i'],
             help="Enable showing password information (entropy, etc).")
     parser.add_argument('-s',
-            default='-',
+            default=flag_defaults['s'],
             metavar="SEPARATOR",
             help="Delimit words with a given character/string.")
     parser.add_argument('-l',
-            default=100,
+            default=flag_defaults['l'],
             type=int,
             metavar="LENGTH",
             help="The maximum word length. Words must be at or below this length.")
     parser.add_argument('-c',
-            default=1,
+            default=flag_defaults['c'],
             type=int,
             metavar="COUNT",
             help="Number of passwords to generate. Defaults to 1.")
     parser.add_argument('-v',
             action='store_true',
-            default=False,
+            default=flag_defaults['v'],
             help="Display version information.")
     return parser
 
 def main():
     parser = createParser()
     flags = parser.parse_args()
+
+    if flags.l < 1:
+        sys.stderr.write("ERROR: Maxmimum word length is less than 1.\n")
+        return
+    if flags.c < 1:
+        sys.stderr.write("ERROR: Password count is less than 1.\n")
+        return
+
     if flags.v:
         print "Version: " + VERSION_NUMBER
         return
+
     pGen = iter(PasswordGen(flags))
     # Print the password without adding a \n or space.
     password_count = int(flags.c)
@@ -186,11 +259,10 @@ def main():
         pGenEnumerate = iter(Enumerator(pGen))
         for i in xrange(password_count):
             sys.stdout.write(pGenEnumerate.next() + ("\n" if i != flags.c - 1 else ""))
-    else:
-        sys.stderr.write("ERROR: Password count is less than 1.\n")
     # Print newline if newline not disabled.
     if not flags.n:
         print ""
+
     # Make sure the password gets printed before info.
     sys.stdout.flush()
     if flags.i:
